CON

  _CLKMODE = XTAL1 + PLL16X
  _XINFREQ = 5_000_000  
  
  #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR
  #14, GOTOX, GOTOY, CLS

  BUFFER_SIZE = 100  

  'Transmission and reception constants
  STX = 2 'Start Of Text
  ETX = 3 'End Of Text
  ENQ = 5 'Enquire the RESULT of the transmission
  ACK = 6 'Positive RESULT of a successful transmission
  NAK = 21 'Negative RESULT of an unsuccessful transmission
  US = 31 'Data block seperator which is used for parsing

VAR
  LONG stack[32]
  BYTE MessageBuffer[BUFFER_SIZE]
  BYTE CRCBuffer[BUFFER_SIZE]
    
OBJ
  Terminal : "FullDuplexSerial"
  Sender : "FullDuplexSerial"
  Receiver : "FullDuplexSerial"

PUB Main

  Terminal.Start(31, 30, 0, 115_200)
  Receiver.Start(0, 1, 0, 9600)
  
  WAITCNT(CLKFREQ * 2 + CNT)

  Terminal.Tx(CLS)
  Terminal.Rxflush

  COGNEW(SendDataStringWithCRC, @stack)

  MonitorForIncomingData  

PUB MonitorForIncomingData | Index, Char, CRC

  'Monitor for Start Of Text (Incoming Data) 
  REPEAT UNTIL Receiver.Rx == STX
  'Start Of Text has been found

  'Start filling the MessageBuffer
  Terminal.Str(string("MessageBuffer filling has started!", CR))

  'Clear the MessageBuffer and fill it with zeros 
  BYTEFILL(@MessageBuffer, 0, BUFFER_SIZE)

  'Set buffer array index to zero  
  Index := 0

  'Keep filling the MessageBuffer until
  'a unit seperator is found 
  REPEAT WHILE (Char := Receiver.Rx) <> US
    MessageBuffer[Index] := Char
    Index++

  'MessageBuffer filling complete
  Terminal.Str(string("MessageBuffer filling has ended!", CR))

  REPEAT UNTIL Receiver.Rx <> US
  Terminal.Str(string("Unit seperator has been bypassed", CR))  

  'Start filling the CRCBuffer
  Terminal.Str(string("CRCBuffer filling has started!", CR))

  'Clear the CRCBuffer and fill it with zeros 
  BYTEFILL(@CRCBuffer, 0, BUFFER_SIZE)

  'Set buffer array index to zero  
  Index := 0

  'Keep filling the CRCBuffer until
  'a unit seperator is found 
  REPEAT WHILE (Char := Receiver.Rx) <> US
    CRCBuffer[Index] := Char
    Index++

  'CRCBuffer filling complete
  Terminal.Str(string("CRCBuffer filling has ended!", CR))

  REPEAT UNTIL Receiver.Rx == ETX
  Terminal.Str(string("Unit seperator has been bypassed and ETX has been found", CR))

  'Display the data contained within the Buffer
  Terminal.Str(@MessageBuffer)
  Terminal.Tx(13)
  Terminal.Str(@CRCBuffer)
  Terminal.Tx(13)

  CRC := ComputeCRC(@MessageBuffer, Index)
  
  Terminal.Dec(CRC)
  Terminal.Tx(13)  

PUB SendDataStringWithCRC | DataString, DataSize, CRC

  Sender.Start(1, 0, 0, 9600)
  WAITCNT(CLKFREQ * 4 + CNT)

  DataString := STRING("The big, bad, burly, brown bear, barely bit the beautifully bestowed bosoomly blonde.")

  DataSize := STRSIZE(DataString)

  CRC := ComputeCRC(@DataString, DataSize)  

  'Send Start Of Text message to the Receiver
  Sender.Tx(STX)
  { Format and send main data content between here and the next comment }

  Sender.Str(DataString)
  Sender.Tx(US)
  Sender.Dec(CRC)
  Sender.Tx(US)

  { Main data content should have been formatted and sent by this point }
  'Send End Of Text message to the Receiver
  Sender.Tx(ETX)

PUB ComputeCRC(DataPointer, SizeOfBytes)
  
  REPEAT SizeOfBytes
  
    RESULT ^= (BYTE[DataPointer++] << 8)
    
    REPEAT 8
    
      RESULT <<= 1
      
      IF RESULT & $10000
      
        RESULT ^= $1021
        
  RESULT &= $FFFF